@@ -145,6 +145,11 @@ def main(a):
     action="store_true",
     help="Reset pandem flight related information to system defaults", 
   )
+  reset_parser.add_argument(
+    "--owid",
+    action="store_true",
+    help="Reset pandem owid related information to system defaults"
+  )
 
   reset_parser.set_defaults(func = do_reset)
 
@@ -221,21 +226,44 @@ def do_start(args, *other):
     return None
   
   if args.restart_job > 0:
-    orchestrator_ref = Orchestration.start(settings, start_acquisition = False, retry_failed = False, restart_job = args.restart_job, retry_active = True, force_acquire = args.force_acquire, no_nlp = args.no_nlp)
+    orchestrator_ref = Orchestration.start(
+      settings, 
+      start_acquisition = False, 
+      retry_failed = False, 
+      restart_job = args.restart_job, 
+      retry_active = True, 
+      force_acquire = args.force_acquire, 
+      no_nlp = args.no_nlp
+    )
     orch = orchestrator_ref.proxy()
   elif args.limit_collection is not None:
-    orchestrator_ref = Orchestration.start(settings, start_acquisition = False, retry_failed = args.retry_failed, restart_job = args.restart_job, retry_active = not args.not_retry_active, force_acquire = args.force_acquire, no_nlp = args.no_nlp)
+    orchestrator_ref = Orchestration.start(
+      settings, start_acquisition = False, 
+      retry_failed = args.retry_failed, 
+      restart_job = args.restart_job, 
+      retry_active = 
+      not args.not_retry_active, 
+      force_acquire = args.force_acquire, 
+      no_nlp = args.no_nlp
+    )
     orch = orchestrator_ref.proxy()
     storage_proxy = orch.get_actor("storage").get().proxy()
     dls_files = storage_proxy.list_files('source-definitions').get()
     dls_dicts = [storage_proxy.read_file(file_name['path']).get() for file_name in dls_files if file_name['path'].endswith(".json")]
-    
     for source_name in args.limit_collection.split(","):
-      dls = list(filter(lambda dls: dls['scope']['source'] == source_name, dls_dicts))[0]
-      acquisition_proxy = orch.get_actor(f"acquisition_{dls['acquisition']['channel']['name']}").get().proxy()
-      acquisition_proxy.add_datasource(dls, args.force_acquire)
+      dlss = list(filter(lambda dls: dls['scope']['source'] == source_name or (source_name in dls['scope']['tags'] and (dls['acquisition'].get("active") != False)), dls_dicts))
+      for dls in dlss:
+        acquisition_proxy = orch.get_actor(f"acquisition_{dls['acquisition']['channel']['name']}").get().proxy()
+        acquisition_proxy.add_datasource(dls, args.force_acquire)
   else:
-    orchestrator_ref = Orchestration.start(settings, start_acquisition = not args.no_acquire, retry_failed = args.retry_failed, retry_active = not args.not_retry_active, force_acquire = args.force_acquire, no_nlp = args.no_nlp)
+    orchestrator_ref = Orchestration.start(
+      settings, 
+      start_acquisition = not args.no_acquire, 
+      retry_failed = args.retry_failed, 
+      retry_active = not args.not_retry_active, 
+      force_acquire = args.force_acquire, 
+      no_nlp = args.no_nlp
+    )
     orch = orchestrator_ref.proxy()
 
   # launching pandem2source app
@@ -281,20 +309,54 @@ def do_reset(args, *other):
     admin.reset_source("covid19-template-local-regions-THL")
   if args.covid19_datahub or args.restore_factory_defaults:
     admin.reset_source("covid19-datahub")
+    admin.reset_source("covid19-datahub-AUT")
+    admin.reset_source("covid19-datahub-BEL")
+    admin.reset_source("covid19-datahub-BGR")
+    admin.reset_source("covid19-datahub-HRV")
+    admin.reset_source("covid19-datahub-CYP")
+    admin.reset_source("covid19-datahub-CZE")
+    admin.reset_source("covid19-datahub-DNK")
+    admin.reset_source("covid19-datahub-EST")
+    admin.reset_source("covid19-datahub-FIN")
+    admin.reset_source("covid19-datahub-FRA")
+    admin.reset_source("covid19-datahub-DEU")
+    admin.reset_source("covid19-datahub-GRC")
+    admin.reset_source("covid19-datahub-HUN")
+    admin.reset_source("covid19-datahub-IRL")
+    admin.reset_source("covid19-datahub-ITA")
+    admin.reset_source("covid19-datahub-LVA")
+    admin.reset_source("covid19-datahub-LTU")
+    admin.reset_source("covid19-datahub-LUX")
+    admin.reset_source("covid19-datahub-MLT")
+    admin.reset_source("covid19-datahub-NLD")
+    admin.reset_source("covid19-datahub-POL")
+    admin.reset_source("covid19-datahub-PRT")
+    admin.reset_source("covid19-datahub-ROU")
+    admin.reset_source("covid19-datahub-SVK")
+    admin.reset_source("covid19-datahub-SVN")
+    admin.reset_source("covid19-datahub-ESP")
+    admin.reset_source("covid19-datahub-SWE")
+    admin.reset_source("covid19-datahub-GBR")
+
   if args.ecdc_covid19_simulated or args.restore_factory_defaults:
     admin.reset_source("ecdc-covid19-age-group-variants")
+    admin.reset_source("ecdc-covid19-weekly-hospital-occupancy-variants")
+    admin.reset_source("ecdc-covid19-genomic-data")
   if args.ecdc_covid19 or args.restore_factory_defaults:
+    admin.reset_source("ecdc-covid19-vaccination")
     admin.reset_source("ecdc-covid19-variants")
     admin.reset_source("ecdc-covid19-age-group")
     admin.reset_source("ecdc-covid19-measures")
     admin.reset_source("ecdc-covid19-daily")
+    admin.reset_source("ecdc-covid19-weekly-hospital-occupancy")
   if args.serotracker or args.restore_factory_defaults:
     admin.reset_source("geonames-countries")    
     admin.reset_source("serotracker")    
   if args.ecdc_atlas or args.restore_factory_defaults:
     admin.reset_source("ecdc-atlas-influenza")    
   if args.influenzanet or args.restore_factory_defaults:
-    admin.reset_source("influenza-net")    
+    admin.reset_source("influenza-net")
+    admin.reset_source("influenza-net-visits")
   if args.twitter or args.restore_factory_defaults:
     admin.reset_source("twitter")    
   if args.medisys or args.restore_factory_defaults:
@@ -302,6 +364,8 @@ def do_reset(args, *other):
   if args.flights or args.restore_factory_defaults:
     admin.reset_source("ourairports")
     admin.reset_source("opensky-network-coviddataset")
+  if args.owid or args.restore_factory_defaults:
+    admin.reset_source("owid-covid19-excess-mortality")
 
 def do_list(args, *other):
   if args.sources or args.missing_sources or args.package_sources or args.missing_package_sources:
