@@ -1,3 +1,5 @@
+from dataclasses import dataclass
+from datetime import datetime
 import tornado.escape
 import tornado.ioloop
 import tornado.web
@@ -12,13 +14,17 @@ import pandas as pd
 import tornado.httpserver
 import tornado.netutil
 import tornado.websocket
+import ast
 from tornado_swagger.components import components
 from tornado_swagger.setup import setup_swagger
-import re
 import json
 
 l = logging.getLogger("pandem-api")
 
+@dataclass(frozen=True)
+class ConstantsNamespace:
+  JSON_EXT = ".json"
+
 
 class apiREST(worker.Worker):
     __metaclass__ = ABCMeta 
@@ -32,6 +38,7 @@ class apiREST(worker.Worker):
         self._storage_proxy = self._orchestrator_proxy.get_actor('storage').get().proxy()
         self._variables_proxy = self._orchestrator_proxy.get_actor('variables').get().proxy()
         self._pipeline_proxy = self._orchestrator_proxy.get_actor('pipeline').get().proxy()
+        self._api_proxy = self._orchestrator_proxy.get_actor('api').get().proxy()
         try:
             l.debug("Starting HTTP server")
             sockets = tornado.netutil.bind_sockets(self._api_port) 
@@ -39,6 +46,7 @@ class apiREST(worker.Worker):
                 storage_proxy=self._storage_proxy,
                 variables_proxy = self._variables_proxy,
                 pipeline_proxy = self._pipeline_proxy,
+                api_proxy = self._api_proxy,
                 sockets=sockets
             )
             self.server.start()
@@ -51,11 +59,12 @@ class apiREST(worker.Worker):
 
 class HttpServer(threading.Thread):
 
-    def __init__(self, storage_proxy, variables_proxy, pipeline_proxy, sockets): 
+    def __init__(self, storage_proxy, variables_proxy, pipeline_proxy, api_proxy, sockets): 
         super().__init__()
         self.storage_proxy = storage_proxy
         self.variables_proxy = variables_proxy
         self.pipeline_proxy = pipeline_proxy
+        self.api_proxy = api_proxy
         self.sockets = sockets
         self.app = None
         self.server = None
@@ -67,7 +76,7 @@ class HttpServer(threading.Thread):
             # start Tornado in a another thread than the main thread, we must
             # explicitly create an asyncio loop for the current thread.
             asyncio.set_event_loop(asyncio.new_event_loop())
-        self.app = Application(self.storage_proxy, self.variables_proxy, self.pipeline_proxy)
+        self.app = Application(self.storage_proxy, self.variables_proxy, self.pipeline_proxy, self.api_proxy)
         self.server = tornado.httpserver.HTTPServer(self.app)
         self.server.add_sockets(self.sockets)
         self.io_loop = tornado.ioloop.IOLoop.current()
@@ -81,7 +90,7 @@ class HttpServer(threading.Thread):
 class SourcesHandler(tornado.web.RequestHandler):
     def initialize(self, storage_proxy):
         self.storage_proxy = storage_proxy
-    def get(self):
+    async def get(self):
         """
         ---
         tags:
@@ -103,9 +112,9 @@ class SourcesHandler(tornado.web.RequestHandler):
                   schema:
                     type: string
         """
-        df_sources = self.storage_proxy.read_db('source').get()
-        df_jobs = self.storage_proxy.read_db('job').get()
-        df_issues = self.storage_proxy.read_db('issue').get()
+        df_sources = await self.storage_proxy.read_db('source')
+        df_jobs = await self.storage_proxy.read_db('job')
+        df_issues = await self.storage_proxy.read_db('issue')
         tags = {}
         sources = {s['name']:s for s in (df_sources.to_dict('records') if df_sources is not None else [])} 
         jobs = df_jobs.to_dict('records') if df_jobs is not None else []
@@ -158,7 +167,7 @@ class JobsBySourceHandler(tornado.web.RequestHandler):
     def initialize(self, storage_proxy, pipeline_proxy):
         self.storage_proxy = storage_proxy
         self.pipeline_proxy = pipeline_proxy
-    def get(self):
+    async def get(self):
         """
         ---
         tags:
@@ -188,8 +197,8 @@ class JobsBySourceHandler(tornado.web.RequestHandler):
                     type: string
         """
         source = self.get_argument('source', default = None)
-        df_issues = self.storage_proxy.read_db('issue').get()
-        df_jobs = self.storage_proxy.read_db('job').get()
+        df_issues = await self.storage_proxy.read_db('issue')
+        df_jobs = await self.storage_proxy.read_db('job')
         jobs_list = []
         for j in df_jobs.to_dict('records'):
           dls = j['dls_json']
@@ -213,7 +222,7 @@ class JobsBySourceHandler(tornado.web.RequestHandler):
         response = {'jobs' : jobs_list}
         self.write(response)
 
-    def delete(self):
+    async def delete(self):
         """
         ---
         tags:
@@ -237,13 +246,13 @@ class JobsBySourceHandler(tornado.web.RequestHandler):
                     type: string
         """
         job_id = self.get_argument('job_id', default = None)
-        self.pipeline_proxy.fail_job(int(job_id), delete_job = True).get()
+        await self.pipeline_proxy.fail_job(int(job_id), delete_job = True)
         self.write(job_id)
 
 class IssuesHandler(tornado.web.RequestHandler):
     def initialize(self, storage_proxy):
         self.storage_proxy = storage_proxy
-    def get(self):
+    async def get(self):
         """
         ---
         tags:
@@ -281,18 +290,18 @@ class IssuesHandler(tornado.web.RequestHandler):
         source = self.get_argument('source', default = None)
         job = self.get_argument('job_id', default = None)
 
-        sources = list(self.storage_proxy.read_db('source').get()['name'])
+        sources = list((await self.storage_proxy.read_db('source'))['name'])
         if source is not None and source not in sources:
             self.set_status(400)
             return self.write("Invalid or not found source")
         else:
-            df_jobs = self.storage_proxy.read_db('job').get()
+            df_jobs = await self.storage_proxy.read_db('job')
             if source is not None:
               df_jobs = df_jobs[df_jobs['source']==source]
             if job is not None:
               df_jobs = df_jobs[df_jobs['id'].astype(int)==int(job)]
 
-            df_issues = self.storage_proxy.read_db('issue').get()
+            df_issues = await self.storage_proxy.read_db('issue')
             if df_jobs is not None:
               dlss = {j['source']:j['dls_json'] for j in df_jobs.to_dict('records')}
             else :
@@ -315,7 +324,7 @@ class IssuesHandler(tornado.web.RequestHandler):
 class SourceDetailsHandler(tornado.web.RequestHandler):
     def initialize(self, storage_proxy):
         self.storage_proxy = storage_proxy
-    def get(self):
+    async def get(self):
         """
         ---
         tags:
@@ -346,8 +355,10 @@ class SourceDetailsHandler(tornado.web.RequestHandler):
         """
         source = self.get_argument('source', default = None)
         sources_path = util.pandem_path("files", "source-definitions")
-        dls_paths = self.storage_proxy.list_files(sources_path).get()
-        dlss = [self.storage_proxy.read_file(f['path']).get() for f in dls_paths if f['path'].endswith(".json")]
+        dls_paths = await self.storage_proxy.list_files(sources_path)
+        constants = ConstantsNamespace()
+        dlss = [(await self.storage_proxy.read_file(f['path'])) for f in dls_paths if f['path'].endswith(constants.JSON_EXT)]
+        
         defs = {dls["scope"]["tags"][0]+ " - "+ dls["scope"]["source"] if "tags" in dls["scope"] and len(dls["scope"]["tags"]) > 0 else dls["scope"]["source"]:dls for dls in dlss}
         if source is not None:
           defs = {k:v for k, v in defs.items() if k == source}
@@ -358,7 +369,7 @@ class VariableListHandler(tornado.web.RequestHandler):
     def initialize(self, storage_proxy, variables_proxy):
         self.storage_proxy = storage_proxy
         self.variables_proxy = variables_proxy
-    def get(self):
+    async def get(self):
         """
         ---
         tags:
@@ -384,14 +395,15 @@ class VariableListHandler(tornado.web.RequestHandler):
 
 
         var_path = util.pandem_path("files", "variables")
-        var_paths = self.storage_proxy.list_files(var_path, include_files = False, include_dirs = True, recursive = False).get()
-        var_dic = self.variables_proxy.get_variables().get()
+        var_paths = await self.storage_proxy.list_files(var_path, include_files = False, include_dirs = True, recursive = False)
+        var_dic = await self.variables_proxy.get_variables()
         
         pub_vars = {v['path'] for v in var_paths }
 
         sources_path = util.pandem_path("files", "source-definitions")
-        dls_paths = self.storage_proxy.list_files(sources_path).get()
-        dlss = [self.storage_proxy.read_file(f['path']).get() for f in dls_paths if f['path'].endswith(".json")]
+        dls_paths = await self.storage_proxy.list_files(sources_path)
+        constants = ConstantsNamespace()
+        dlss = [(await self.storage_proxy.read_file(f['path'])) for f in dls_paths if f['path'].endswith(constants.JSON_EXT)]
         used_vars = {col["variable"] for dls in dlss for col in dls['columns'] if "variable" in col}
         found_vars = {}
         for v in var_dic:
@@ -410,7 +422,7 @@ class TimeSerieHandler(tornado.web.RequestHandler):
     def initialize(self, storage_proxy, variables_proxy):
         self.storage_proxy = storage_proxy
         self.variables_proxy = variables_proxy
-    def post(self):
+    async def post(self):
         """
         ---
         tags:
@@ -433,12 +445,11 @@ class TimeSerieHandler(tornado.web.RequestHandler):
                   schema:
                     type: string
         """
-        storage_proxy = self.storage_proxy
         variables_proxy = self.variables_proxy
-        var_dic = variables_proxy.get_variables().get()
+        var_dic = await variables_proxy.get_variables()
         query = tornado.escape.json_decode(self.request.body)
+        resp = {}
         if query is not None:
-          ts = variables_proxy.get_timeseries().get()
           comb = [(k, v) for (k,v) in query.items() if v is not None and k != 'indicator' and k != 'source']
           comb.sort(key = lambda v: v[0])
           source = query["source"]
@@ -446,9 +457,8 @@ class TimeSerieHandler(tornado.web.RequestHandler):
           variable = var_dic[indicator]["variable"]
           datevars = [v for v, varinfo in var_dic.items() if varinfo['type']=='date' and varinfo['variable']==v]
 
-          data = variables_proxy.lookup([variable], source = source, combinations = [tuple(comb)], filter = {d:None for d in datevars}).get()
+          data = await variables_proxy.lookup([variable], source = source, combinations = [tuple(comb)], filter = {d:None for d in datevars})
           # making an aggregation at day level
-          resp = {}
           for key, values in data.items():
             for var, tuples in values.items():
               for t in tuples:
@@ -460,7 +470,7 @@ class TimeSerieHandler(tornado.web.RequestHandler):
                     #resp[(date, datevar)].update({k:v for k,v in keys})
                   if var not in resp[(date, datevar)]:
                     resp[(date, datevar)]["indicator"] = indicator
-                    resp[(date, datevar)]["value"] = value
+                    resp[(date, datevar)]["value"] = value if value not in [np.inf, -np.inf] else None
                   else :
                     # TODO: change the aggregation function depending on the unit
                     resp[(date, datevar)]["value"] = resp[(date, datevar)]["value"] + value
@@ -472,9 +482,7 @@ class TimeSeriesHandler(tornado.web.RequestHandler):
     def initialize(self, storage_proxy, variables_proxy):
         self.storage_proxy = storage_proxy
         self.variables_proxy = variables_proxy
-        self._timeseries_hash = ''
-        self._timeseries_cache = ''
-    def get(self):
+    async def get(self):
         """
         ---
         tags:
@@ -483,6 +491,52 @@ class TimeSeriesHandler(tornado.web.RequestHandler):
         description: Get time serie data
         operationId: getTimeSerie
         parameters:
+          - name: source
+            in: query
+            description: source name
+            required: false
+            schema:
+              type: string
+          - name: indicator
+            in: query
+            description: indicator name
+            required: false
+            schema:
+              type: string
+          - name: source_table
+            in: query
+            description: source table name
+            required: false
+            schema:
+              type: string
+          - name: geo_code
+            in: query
+            description: geographic location code
+            required: false
+            schema:
+              type: string
+          - name: key
+            in: query
+            description: Adds time series key to call timeserue endpoint
+            required: false
+            schema:
+              type: bool
+          - name: data
+            in: query
+            description: Adds data points to times series
+            required: false
+            schema:
+              type: bool
+          - name: limit
+            description: Maximum number of timeseries to return default to 0 = all time series
+            required: false
+            schema:
+              type: int
+          - name: offset
+            description: Number of timeseries to skip
+            required: false
+            schema:
+              type: int
         responses:
             '200':
               description: Data of time serie
@@ -499,105 +553,171 @@ class TimeSeriesHandler(tornado.web.RequestHandler):
         """
         storage_proxy = self.storage_proxy
         variables_proxy = self.variables_proxy
-        # If no changes have been performed on timeseries and we have a let's try to reuse last value
-        if self._timeseries_cache != '' and self._timeseries_hash != '' and self._timeseries_hash == self.variables_proxy.timeseries_hash.get():
-            self.write(self._timeseries_cache)
+        f_source = self.get_argument('source', default = None)
+        f_source_table = self.get_argument('source_table', default = None)
+        f_indicator = self.get_argument('indicator', default = None)
+        f_geo_code = self.get_argument('geo_code', default = None)
+        f_data = ast.literal_eval(self.get_argument('data', default = 'False'))
+        f_key = ast.literal_eval(self.get_argument('key', default = 'False'))
+        f_limit = ast.literal_eval(self.get_argument('limit', default = "0"))
+        f_offset = ast.literal_eval(self.get_argument('offset', default = "0"))
+        
+        # calculating time series
+        constants = ConstantsNamespace()
+        dlss = [(await storage_proxy.read_file(f["path"])) 
+          for f in (await storage_proxy.list_files(util.pandem_path("files", "source-definitions"))) 
+            if f["name"].endswith(constants.JSON_EXT)
+              and (f_source_table is None or f["name"].endswith(f'{f_source_table}{constants.JSON_EXT}'))
+        ]
+        if f_source is not None:
+           dlss = [dls for dls in dlss if f_source == (dls["scope"]["tags"][0] if "tags" in dls["scope"] and len(dls["scope"]["tags"]) > 0 else dls["scope"]["name"])]
+        
+        source_names = {dls["scope"]["source"]:dls["scope"]["tags"][0] if "tags" in dls["scope"] and len(dls["scope"]["tags"]) > 0 else dls["scope"]["name"] for dls in dlss}
+        reference_users = {dls["scope"]["source"]:dls["scope"]["reference_user"] if "reference_user" in dls["scope"] else "" for dls in dlss}
+        data_quality = {dls["scope"]["source"]:dls["scope"]["data_quality"] if "data_quality" in dls["scope"] else "" for dls in dlss}
+        source_descriptions = {dls["scope"]["source"]:dls["scope"]["source_description"] if "source_description" in dls["scope"] else "" for dls in dlss}
+        var_dic = await variables_proxy.get_variables()
+        
+        # getting referential aliases
+        ref_labels = {code:k for k, varinfo in var_dic.items() if varinfo["type"] == "referential_label" for code in varinfo["linked_attributes"]}
+        ref_attrs = {}
+        ref_link = {}
+        for k, varinfo in var_dic.items(): 
+          if varinfo["type"] in ["characteristic", "referential_parent" ] and varinfo['linked_attributes'] is not None: 
+            for code in varinfo["linked_attributes"]:
+              if code not in ref_attrs:
+                ref_attrs[code]= []
+              ref_attrs[code].append(k)
+              ref_link[k]=code
+              
+        code_labels = {}
+        code_attrs = {}
+        no_labels = ["indicator__description", "source__reference_user", "source__source_description"]
+        ts = await variables_proxy.get_timeseries()
+        ts_values = [{k:v for k, v in key} for key in ts.keys() if
+            # applying filters if any
+            (f_source is None or len({k:v for k, v in key if k == "source" and v in source_names}) > 0)
+            and (f_source_table is None or len({k:v for k, v in key if k == "source" and v == f_source_table}) > 0)
+            and (f_indicator is None or len({k:v for k, v in key if k == "indicator" and v == f_indicator}) > 0)
+            and (f_geo_code is None or len({k:v for k, v in key if k == "geo_code" and v == f_geo_code}) > 0)
+        ]
+        refs_read = set()
+        
+        # Adding nice to have information on time series
+        if f_limit <= 0:
+          limit = len(ts_values)
         else:
-            # recalculating time series
-            dlss = [storage_proxy.read_file(f["path"]).get() for f in storage_proxy.list_files(util.pandem_path("files", "source-definitions")).get() if f["name"].endswith(".json")]
-            source_names = {dls["scope"]["source"]:dls["scope"]["tags"][0] if "tags" in dls["scope"] and len(dls["scope"]["tags"]) > 0 else dls["scope"]["name"] for dls in dlss}
-            reference_users = {dls["scope"]["source"]:dls["scope"]["reference_user"] if "reference_user" in dls["scope"] else "" for dls in dlss}
-            data_quality = {dls["scope"]["source"]:dls["scope"]["data_quality"] if "data_quality" in dls["scope"] else "" for dls in dlss}
-            source_descriptions = {dls["scope"]["source"]:dls["scope"]["source_description"] if "source_description" in dls["scope"] else "" for dls in dlss}
-            var_dic = variables_proxy.get_variables().get()
-            
-            # getting referential aliases
-            ref_labels = {code:k for k, varinfo in var_dic.items() if varinfo["type"] == "referential_label" for code in varinfo["linked_attributes"]}
-            ref_attrs = {}
-            ref_link = {}
-            for k, varinfo in var_dic.items(): 
-              if varinfo["type"] in ["characteristic", "referential_parent" ] and varinfo['linked_attributes'] is not None: 
-                for code in varinfo["linked_attributes"]:
-                  if code not in ref_attrs:
-                    ref_attrs[code]= []
-                  ref_attrs[code].append(k)
-                  ref_link[k]=code
-                  
-            code_labels = {}
-            code_attrs = {}
-            no_labels = ["indicator__description", "source__reference_user", "source__source_description"]
-            ts = variables_proxy.get_timeseries().get()
-            self._timeseries_hash = variables_proxy.timeseries_hash.get()
-            ts_values = [{k:v for k, v in key} for key in ts.keys()]
-            # Adding nice to have information on time series
-            for values in ts_values:
-              # Adding related attribute characteristics
-              for var, value in list(values.items()):
-                if var in ref_attrs:
-                  # Loading referential attrs if present 
-                  for attr in  ref_attrs[var]:
-                    if attr not in code_attrs:
-                       link = variables_proxy.get_referential(var).get()
-                    if link is not None:
-                      code_attrs[attr] = {t['attr'][var]:t['attrs'][attr] for t in link if 'attrs' in t and 'attr' in t and attr in t['attrs'] and var in t['attr']}
-                    # Taking label from referential
-                    if attr in code_attrs and value in code_attrs[attr]:
-                      values[f"ref__{attr}"] = code_attrs[attr][value]
+          limit = f_limit
+        if f_offset <= 0:
+          offset = 0
+        else:
+          offset = f_offset
 
-              # Adding frienly labels
-              for var, value in list(values.items()):
-                var = var.split("__")[-1]
-                if var in ref_labels:
-                  # if variable is a characteristic with a link to a referential we have to use the referential as query table
-                  if var not in ref_link:
-                    label_name = f"{var}_label"
-                    ref_var = var 
-                  else:
-                    ref_var = ref_link[var]
-                    label_name = f"ref__{var}_label"
-                  
-                  # Loading referential labels if present 
-                  label = ref_labels[ref_var]
-                  if ref_var not in code_labels:
-                    labels = variables_proxy.get_referential(label).get()
-                    if labels is not None:
-                      code_labels[ref_var] = {t['attrs'][ref_var]:t['attr'][label] for t in labels if 'attrs' in t and 'attr' in t and label in t['attr'] and ref_var in t['attrs']}
-                  # Taking label from referential if exists
-                  if ref_var in code_labels and value in code_labels[ref_var]:
-                    values[label_name] = code_labels[ref_var][value]
-                  # Taking code as value if not Nont 
-                  elif value is not None:
-                    values[label_name] = str(value) 
-                  else:
-                    values[label_name] = None
-                  # Making case correction when possible
-                  #if values[label_name] is not None and len(values[label_name])>2: #and ref_var not in no_labels:
-                  #  values[label_name] = " ".join([(word[0].upper()+word[1:].lower() if len(word)>2 else word)  for word in re.split("_| |\\-", values[label_name])])
+        ret_values = ts_values[offset:(offset+limit)]
 
-              # Adding indicator associated values
-              if "indicator" in values:
-                values["indicator__family"] = var_dic[values["indicator"]]["data_family"]
-                values["indicator__description"] = var_dic[values["indicator"]]["description"]
-                values["indicator__unit"] = var_dic[values["indicator"]]["unit"]
-              # Adding source (DLS) associated values
-              if "source" in values:
-                values["source__table"] = values["source"]
-                values["source__source_name"] = source_names[values["source"]]
-                values["source__reference_user"] = reference_users[values["source"]]
-                values["source__source_description"] = source_descriptions[values["source"]]
-                values["source__data_quality"] = data_quality[values["source"]]
+        for values in ret_values:
+          orig_values = values.copy()
+          # Adding related attribute characteristics
+          for var, value in list(values.items()):
+            if var in ref_attrs:
+              # Loading referential attrs if present 
+              for attr in ref_attrs[var]:
+                if (var, attr) not in refs_read:
+                  refs_read.add((var, attr))
+                  if attr not in code_attrs:
+                     link = await variables_proxy.get_referential(var)
+                  if link is not None:
+                    code_attrs[attr] = {t['attr'][var]:t['attrs'][attr] for t in link if 'attrs' in t and 'attr' in t and attr in t['attrs'] and var in t['attr']}
+                # Taking label from referential
+                if attr in code_attrs and value in code_attrs[attr]:
+                  values[f"ref__{attr}"] = code_attrs[attr][value]
+
+          # Adding frienly labels
+          for var, value in list(values.items()):
+            var = var.split("__")[-1]
+            if var in ref_labels:
+              # if variable is a characteristic with a link to a referential we have to use the referential as query table
+              if var not in ref_link:
+                label_name = f"{var}_label"
+                ref_var = var 
+              else:
+                ref_var = ref_link[var]
+                label_name = f"ref__{var}_label"
+              
+              # Loading referential labels if present 
+              label = ref_labels[ref_var]
+              if ref_var not in code_labels:
+                labels = await variables_proxy.get_referential(label)
+                if labels is not None:
+                  code_labels[ref_var] = {t['attrs'][ref_var]:t['attr'][label] for t in labels if 'attrs' in t and 'attr' in t and label in t['attr'] and ref_var in t['attrs']}
+              # Taking label from referential if exists
+              if ref_var in code_labels and value in code_labels[ref_var]:
+                values[label_name] = code_labels[ref_var][value]
+              # Taking code as value if not Nont 
+              elif value is not None:
+                values[label_name] = str(value) 
+              else:
+                values[label_name] = None
+              # Making case correction when possible
+              #if values[label_name] is not None and len(values[label_name])>2: #and ref_var not in no_labels:
+              #  values[label_name] = " ".join([(word[0].upper()+word[1:].lower() if len(word)>2 else word)  for word in re.split("_| |\\-", values[label_name])])
+
+          # Adding indicator associated values
+          if "indicator" in values and values["indicator"] in var_dic:
+            values["indicator__family"] = var_dic[values["indicator"]]["data_family"]
+            values["indicator__description"] = var_dic[values["indicator"]]["description"]
+            values["indicator__unit"] = var_dic[values["indicator"]]["unit"]
+          # Adding source (DLS) associated values
+          if "source" in values:
+            values["source__table"] = values["source"]
+            if values["source"] in source_names:
+              values["source__source_name"] = source_names[values["source"]]
+            if values["source"] in reference_users:
+              values["source__reference_user"] = reference_users[values["source"]]
+            if values["source"] in source_descriptions:
+              values["source__source_description"] = source_descriptions[values["source"]]
+            if values["source"] in data_quality:
+              values["source__data_quality"] = data_quality[values["source"]]
+          # Adding data points if required
+          if f_key:
+            values["key"] = orig_values
+          if f_data and values["indicator"] in var_dic:
+            comb = [(k, v) for k, v in orig_values.items() if v is not None and k != 'indicator' and k != 'source']
+            comb.sort(key = lambda p:p[0])
+            source = values["source__table"]
+            indicator = values["indicator"]
+            variable = var_dic[indicator]["variable"]
+            datevars = [v for v, varinfo in var_dic.items() if varinfo['type']=='date' and varinfo['variable']==v]
+            data = await variables_proxy.lookup([variable], source = source, combinations = [tuple(comb)], filter = {d:None for d in datevars})
+            # making an aggregation at day level
+            resp = {} 
+            for key, vals in data.items():
+              for var, tuples in vals.items():
+                for t in tuples:
+                  value = t["value"]
+                  for datevar, dtime in t["attrs"].items():
+                    date = dtime[0:10]
+                    if (date, datevar) not in resp:
+                      resp[(date, datevar)] = {'date':date, 'date_var':datevar, "key":json.dumps({k:v for k,v in key})}
+                      #resp[(date, datevar)].update({k:v for k,v in keys})
+                    if var not in resp[(date, datevar)]:
+                      resp[(date, datevar)]["indicator"] = indicator
+                      resp[(date, datevar)]["value"] = value if value not in [np.inf, -np.inf] else None
+                    else :
+                      # TODO: change the aggregation function depending on the unit
+                      resp[(date, datevar)]["value"] = resp[(date, datevar)]["value"] + value
+            values["data"] = list(resp.values())
 
 
-            response = {"timeseries":ts_values}
-            self._timeseries_cache = response
-            self.write(response)
+        response = {"timeseries":ret_values, "total":len(ts_values)}
+        self.write(response)
 
 class DatasetHandler(tornado.web.RequestHandler):
     def initialize(self, storage_proxy, variables_proxy):
         self.storage_proxy = storage_proxy
         self.variables_proxy = variables_proxy
 
-    def post(self):
+    async def post(self):
         """
         ---
         tags:
@@ -620,13 +740,12 @@ class DatasetHandler(tornado.web.RequestHandler):
                   schema:
                     type: string
         """
-        storage_proxy = self.storage_proxy
         variables_proxy = self.variables_proxy
-        var_dic = variables_proxy.get_variables().get()
+        var_dic = await variables_proxy.get_variables()
         
         query = tornado.escape.json_decode(self.request.body)
         if query is not None:
-            ts = variables_proxy.get_timeseries().get()
+            ts = await variables_proxy.get_timeseries()
             needed_variables, query = self.__build_needed_variables(query, var_dic)
             filtered_ts = self.__filter_ts(query, needed_variables, ts)
 
@@ -634,8 +753,8 @@ class DatasetHandler(tornado.web.RequestHandler):
             data = {}
             columns = {'source'}
             for ((source, indicator), comb) in filtered_ts.items():
-                d = variables_proxy.lookup([var_dic[indicator]['variable']], source=source,
-                                          combinations=comb, filter={'reporting_period': None}).get()
+                d = await variables_proxy.lookup([var_dic[indicator]['variable']], source=source,
+                                          combinations=comb, filter={'reporting_period': None})
                 data[(source, indicator)] = d
                 columns.add(indicator)
 
@@ -740,6 +859,31 @@ class DatasetHandler(tornado.web.RequestHandler):
                     return rows[row_key]
 
 
+class SlowHandler(tornado.web.RequestHandler):
+    def initialize(self, storage_proxy, variables_proxy, api_proxy):
+        self.storage_proxy = storage_proxy
+        self.variables_proxy = variables_proxy
+        self.api_proxy = api_proxy
+
+    async def post(self):
+      total = 800
+      count = 0
+      for _ in range(0, total):
+        ref = await self.variables_proxy.get_referential("geo_code")
+        if ref is not None:
+          count += len(ref)
+      print(f"Total count is {count}")
+      self.write({'timeseries_length': count})
+
+
+class FastHandler(tornado.web.RequestHandler):
+    def initialize(self, storage_proxy, variables_proxy):
+        self.storage_proxy = storage_proxy
+        self.variables_proxy = variables_proxy
+    
+    def post(self):
+      self.write({'datetime': str(datetime.now())})
+
 @components.schemas.register
 class SourcesModel(object):
     """
@@ -822,7 +966,7 @@ class DatasetModel(object):
 
 class Application(tornado.web.Application):
 
-    def __init__(self, storage_proxy, variables_proxy, pipeline_proxy):
+    def __init__(self, storage_proxy, variables_proxy, pipeline_proxy, api_proxy):
         settings = {"debug": False}
         self._routes = [
           tornado.web.url(r"/jobs", JobsBySourceHandler, {'storage_proxy': storage_proxy, 'pipeline_proxy':pipeline_proxy}),
@@ -832,7 +976,9 @@ class Application(tornado.web.Application):
           tornado.web.url(r"/variable_list", VariableListHandler, {'storage_proxy': storage_proxy, 'variables_proxy':variables_proxy}),
           tornado.web.url(r"/timeseries", TimeSeriesHandler, {'storage_proxy': storage_proxy, 'variables_proxy':variables_proxy}),
           tornado.web.url(r"/timeserie", TimeSerieHandler, {'storage_proxy': storage_proxy, 'variables_proxy':variables_proxy}),
-          tornado.web.url(r"/dataset", DatasetHandler, {'storage_proxy': storage_proxy, 'variables_proxy':variables_proxy})
+          tornado.web.url(r"/dataset", DatasetHandler, {'storage_proxy': storage_proxy, 'variables_proxy':variables_proxy}),
+          tornado.web.url(r"/slow", SlowHandler, {'storage_proxy': storage_proxy, 'variables_proxy': variables_proxy, 'api_proxy': api_proxy}),
+          tornado.web.url(r"/fast", FastHandler, {'storage_proxy': storage_proxy, 'variables_proxy': variables_proxy})
         ]
         setup_swagger(
             self._routes,
